evm.namespace('evm.util');

(function () {
    /**
     * 文字滚动组件
     * <pre>
     * 创建时间：2016-03-12
     * 2016-04-12 v1.1.0
     * 1.配置文件新增wrapClassName属性
     * 2016-06-13 v1.2.0
     * 1.增加html marquee标签支持，解决盒子对js实现滚动有问题时候
     * </pre>
     * @version 1.1.0
     * @author wuqq@evmtv.com
     * @param {Object} [options] 配置对象
     * @param {Number} [options.gap] 当useMarqueeTag为false，循环滚动的首尾的间距，默认为20px
     * @param {Number} [options.pauseTime] 当useMarqueeTag为false，完成一次滚动后暂停的时间，单位为ms，默认为0ms
     * @param {Number} [options.scrollAmount] 当useMarqueeTag为false，用于控制滚动速度，默认为2
     * @param {Number} [options.startPauseTime] 当useMarqueeTag为false，第一次开始滚动前停留时间，单位为ms，默认为1000ms
     * @param {Number} [options.fontSize] 滚动文字的大小，默认为22px
     * @param {Number} options.width 滚动框的宽度
     * @param {Number} [options.height] 滚动框的宽度，默认为30px
     * @param {String} [options.wrapClassName] 滚动文字的初始包含元素的className，默认为"inner"
     * @param {Boolean} [options.useMarqueeTag} 是否使用marquee标签构建滚动
     * @constructor
     * @example
     * //页面需要引用js
     * /swatapps/ape/util/marquee/main.min.js
     *
     * //初始化滚动组件对象
     * var phoneMarqueeManager = evm.util.marquee({
     *       fontSize: 18,
     *       width: 250,
     *       height: 18,
     *       wrapClassName: 'phone',
     *       useMarqueeTag: true
     *   });
     * //开始滚动
     * //一般在元素获取焦点时候发起滚动
     * //注：phoneElem.innerHTML为非滚动显示的文字
     * //this.getActiveData().phone为全部文字
     * phoneMarqueeManager.startScroll(phoneElem, phoneElem.innerHTML, this.getActiveData().phone);
     *
     * //停止滚动
     * phoneMarqueeManager.stopScroll();
     */
    evm.util.Marquee = function (options) {
        //默认配置
        var defaults = {
            gap: 20,
            pauseTime: 0,
            scrollAmount: 2,
            startPauseTime: 1000,
            fontSize: 22,
            width: 0,
            height: 30,
            prefix: 'evm-marquee',
            wrapClassName: 'inner',
			useMarqueeTag: false,
			isCircle: true
        };
        evm.extend(this, defaults, options);

        //私有属性
        this._timerId = null;
        this._scrollOnceTimerId = null;
        this._boxElm = null;
        this._displayElm = null;
        this._marqueeElm = null;

        this._startPauseTimeId = null;

        this._text = '';
        this._oriText = '';

        //增加滚动框宽度
        this._extWidth = 100;
    };
    evm.util.Marquee.prototype = {
        constructor: evm.util.Marquee,
        _createDOM: function (parentNode, text, oriText) {
            if (arguments.length !== 3) {
                throw new Error('Marquee.prototype.createDOM需要3个参数');
            }
            if (!parentNode) {
                return false;
            }
            else if (encodeURIComponent(text) === encodeURIComponent(oriText)) {
                return false;
            }
            //获取滚动初始化标识属性
            var flag = evm.getElementsByClassName(this.prefix + '-box', parentNode, 'div')[0];
            if (flag) {
                this._boxElm = flag;
                this._displayElm = evm.getElementsByClassName(this.prefix + '-display', parentNode, 'div')[0];
                this._marqueeElm = evm.getElementsByClassName(this.prefix + '-marquee', parentNode, 'div')[0];
                return true;
            }
            else {
                //滚动相关节点未建立
                //清空原来文字
                parentNode.innerHTML = '';
                //创建滚动容器DOM节点
                this._boxElm = evm.createElem('div', {
                    className: this.prefix + '-box'
                }, parentNode);
                evm.css(this._boxElm, {
                    position: 'relative',
                    width: this.width + 'px',
                    height: this.height + 'px',
                    lineHeight: this.height + 'px',
                    overflow: 'hidden',
                    fontSize: this.fontSize
                });

                //展示文字框
                this._displayElm = evm.createElem('div', {
                    className: this.prefix + '-display'
                }, this._boxElm);
                evm.text(this._displayElm, text);

                //滚动元素框
                this._marqueeElm = evm.createElem('div', {
                    className: this.prefix + '-marquee'
                }, this._boxElm);
                evm.css(this._marqueeElm, {
                    position: 'absolute',
                    left: 0,
                    top: 0,
                    width: (oriText.length * this.fontSize + this.gap) * 2 + this._extWidth + 'px',
                    height: this.height + 'px',
                    textAlign: 'left'
                });
                var className = this.prefix + '-text';
                evm.html(this._marqueeElm, '<span class="' + className + '" style="display:inline-block;margin-right:' + this.gap + 'px;">' + oriText + '</span><span class="' + className + '" style="display:inline-block;margin-right:' + this.gap + 'px">' + oriText + '</span>')
                return true;
            }
        },
        _scrollOnce: function (left, lineWidth, step) {
            clearTimeout(this._scrollOnceTimerId);

            var self = this;
            left += step;

            if (left <= -lineWidth) {
				if(self.isCircle) {
					var offset = -lineWidth - left;

					var lineFirst = evm.getElementsByClassName(self.prefix + '-text', self._marqueeElm, 'span')[0];
					self._marqueeElm.appendChild(lineFirst);

					self._marqueeElm.style.left = -offset;
					left = -offset;

					setTimeout(function () {
						self._scrollOnce.call(self, left, lineWidth, step);
					}, self.pauseTime);
				}
                
            }
            else {
                self._marqueeElm.style.left = left + 'px';
                self._scrollOnceTimerId = setTimeout(function () {
                    self._scrollOnce.call(self, left, lineWidth, step);
                }, 40);
            }

        },
        _startScroll: function (parentNode, text, oriText) {
            if (this._boxElm && this._marqueeElm && this._displayElm) {
                //此处是为了部分盒子_stopScroll之前未正确执行完成
                this._stopScroll();
            }
            var self = this;
            var success = this._createDOM(parentNode, text, oriText);
            if (success && this._marqueeElm) {
                evm.css(this._marqueeElm, {
                    display: 'block'
                });

                evm.util.common.logger.debug("left:" + this._marqueeElm.style.left);
                evm.css(this._displayElm, {
                    display: 'none'
                });

                var lineWidth;
                var marqueeTextElm = evm.getElementsByClassName(this.prefix + '-text', this._marqueeElm, 'div')[0];
                this._startPauseTimeId = setTimeout(function () {
                    self._timerId = setInterval(function () {
                        //todo clientWidth可能有兼容性问题
                        var lineWidth = '';
                        try {
                            var lineFirst = evm.getElementsByClassName(self.prefix + '-text', self._marqueeElm, 'span')[0];
                            lineWidth = parseFloat(evm.getStyle(lineFirst, 'width')) || lineFirst.clientWidth;
                        }
                        catch (e) {
                            lineWidth = oriText.length * self.fontSize;
                        }
                        if (lineWidth) {
                            clearInterval(self._timerId);

                            lineWidth += self.gap;
                            var left = parseFloat(evm.getStyle(self._marqueeElm, 'left'));
                            var step = -self.scrollAmount;

                            self._scrollOnce.call(self, left, lineWidth, step);
                        }
                    }, 10);
                }, this.startPauseTime);
            }
        },
        _stopScroll: function () {
            if (this._boxElm && this._marqueeElm && this._displayElm) {
                this._startPauseTimeId && clearTimeout(this._startPauseTimeId);
                this._timerId && clearInterval(this._timerId);
                this._scrollOnceTimerId && clearTimeout(this._scrollOnceTimerId);

                this._startPauseTimeId = null;
                this._scrollOnceTimerId = null;
                this._timerId = null;

                this._marqueeElm && (
                    evm.css(this._marqueeElm, {
                        left: 0
                    })
                );

                evm.css(this._marqueeElm, {
                    display: 'none'
                });
                evm.css(this._displayElm, {
                    display: 'block'
                });

                evm.util.common.logger.debug('<util.marquee.main> _stopScroll() box innerHTML:' + this._displayElm.parentNode.innerHTML);

                //this._boxElm = null;
                //this._marqueeElm = null;
                //this._displayElm = null;
            }
        },
        /**
         * 开始滚动
         * @param {HTMLElement} parentNode 滚动文字的初始包含元素
         * @param {String} text 非滚动时显示的文字（可包含省略符）
         * @param {String} oriText 全部显示时的文字
         */
        startScroll: function (parentNode, text, oriText) {
            if (this.useMarqueeTag) {
                //使用marquee标签处理滚动
                if (!parentNode) {
                    return;
                }
                else if (encodeURIComponent(text) === encodeURIComponent(oriText)) {
                    return false;
                }
                else if (text.toLowerCase().indexOf('<marquee') != -1) {
                    return false;
                }

                this._boxElm = parentNode;

                evm.css(this._boxElm, {
                    position: 'relative',
                    width: this.width + 'px',
                    height: this.height + 'px',
                    lineHeight: this.height + 'px',
                    overflow: 'hidden',
                    fontSize: this.fontSize
                });
                this._boxElm.innerHTML = '';

                //创建marquee标签和内容
                this._marqueeElm = evm.createElem('marquee', {
                    className: this.prefix + '-marquee'
                }, this._boxElm);
                evm.html(this._marqueeElm, oriText);

                this._text = text;
                this._oriText = oriText;
            }
            else {
                this._startScroll(parentNode, text, oriText);
            }
        },
        /**
         * 停止滚动
         */
        stopScroll: function () {
            if (this.useMarqueeTag) {
                if (this._boxElm) {
                    if (this._marqueeElm) {
                        this._marqueeElm.stop && this._marqueeElm.stop();
                        this._marqueeElm.parentNode.removeChild(this._marqueeElm);
                    }
                    evm.html(this._boxElm, this._text);
                    this._marqueeElm = null;
                    this._boxElm = null;
                }
            }
            else {
                this._stopScroll();
            }
        }
    };

    /**
     * @function evm.util.marquee
     * @param options
     * @return {evm.util.Marquee}
     */
    evm.util.marquee = function (options) {
        return new evm.util.Marquee(options);
    }
})();
