/**
 * 文字滚动插件
 * 返回一个Marquee类的实例
 * 依赖/swatapps/ape/evm.js
 * 创建时间：2016-03-12
 * 版本：v1.1.0
 *
 * 2016-04-12 v1.1.0
 * 1.配置文件新增wrapClassName属性
 * 2016-06-13 v1.2.0
 * 1.增加html marquee标签支持，解决盒子对js实现滚动有问题时候
 */
(function (window) {
    function Marquee(options) {
        //默认配置
        var defaults = {
            //循环滚动中间间距
            gap: 20,
            //滚动后停止时间,ms
            pauseTime: 0,
            //控制滚动速度
            scrollAmount: 2,
            //开始滚动前停留时间,ms
            startPauseTime: 1000,
            //滚动文字字体大小
            fontSize: 22,
            //滚动文字框元素的宽度
            width: 0,
            //滚动文字框元素的高度
            height: 30,
            prefix: 'evm-marquee',
            //文字包含元素的className,默认为inner
            wrapClassName: 'inner',
            //是否使用marquee标签
            useMarqueeTag: false
        };
        evm.extend(this, defaults, options);

        //私有属性
        this._timerId = null;
        this._scrollOnceTimerId = null;
        this._boxElm = null;
        this._displayElm = null;
        this._marqueeElm = null;

        this._startPauseTimeId = null;

        this._text = '';
        this._oriText = '';
    }

    /**
     * 初始化滚动DOM
     * @param {object} parentNode 需要滚动文字的父元素
     * @param text 现在展示的文字
     * @param oriText 原始的文字
     * @param {number} fontSize 文字大小
     * @param {number} height 滚动框的高度
     * @returns {*}
     */
    Marquee.prototype._createDOM = function (parentNode, text, oriText) {
        if (arguments.length !== 3) {
            throw new Error('Marquee.prototype.createDOM需要3个参数');
        }
        if (!parentNode) {
            return false;
        }
        else if (encodeURIComponent(text) === encodeURIComponent(oriText)) {
            evm.html(parentNode, text);
            return false;
        }
        //设置滚动初始化标识
        var flag = parentNode.getAttribute('data-marquee');
        if (flag) {
            this._boxElm = evm.getElementsByClassName(this.prefix + '-box', parentNode, 'div')[0];
            this._displayElm = evm.getElementsByClassName(this.prefix + '-display', parentNode, 'div')[0];
            this._marqueeElm = evm.getElementsByClassName(this.prefix + '-marquee', parentNode, 'div')[0];
            return true;
        }
        else {
            //滚动相关节点未建立
            parentNode.setAttribute('data-marquee', 1);
            //清空原来文字，建立自己的DOM结构
            evm.html(parentNode, '');

            //创建滚动包含框
            this._boxElm = evm.createElem('div', {
                className: this.prefix + '-box'
            }, parentNode);
            evm.css(this._boxElm, {
                position: 'relative',
                width: this.width + 'px',
                height: this.height + 'px',
                lineHeight: this.height + 'px',
                overflow: 'hidden',
                fontSize: this.fontSize
            });

            //展示文字框
            this._displayElm = evm.createElem('div', {
                className: this.prefix + '-display'
            }, this._boxElm);
            evm.text(this._displayElm, text);

            //滚动元素框
            this._marqueeElm = evm.createElem('div', {
                className: this.prefix + '-marquee'
            }, this._boxElm);
            evm.css(this._marqueeElm, {
                position: 'absolute',
                left: 0,
                top: 0,
                width: (oriText.length * this.fontSize + this.gap) * 2 + 'px',
                height: this.height + 'px',
                textAlign: 'left'
            });
            var className = this.prefix + '-text';
            evm.html(this._marqueeElm, '<span class="' + className + '" style="display:inline-block;margin-right:' + this.gap + 'px;">' + oriText + '</span><span class="' + className + '" style="display:inline-block;margin-right:' + this.gap + 'px">' + oriText + '</span>')
            return true;
        }
    };
    //滚动一次
    Marquee.prototype._scrollOnce = function (left, lineWidth, step) {
        clearTimeout(this._scrollOnceTimerId);

        var self = this;
        left += step;

        if (left <= -lineWidth) {
            var offset = -lineWidth - left;

            var lineFirst = evm.getElementsByClassName(self.prefix + '-text', self._marqueeElm, 'span')[0];
            self._marqueeElm.appendChild(lineFirst);

            self._marqueeElm.style.left = -offset;
            left = -offset;

            setTimeout(function () {
                self._scrollOnce.call(self, left, lineWidth, step);
            }, self.pauseTime);
        }
        else {
            self._marqueeElm.style.left = left + 'px';
            self._scrollOnceTimerId = setTimeout(function () {
                self._scrollOnce.call(self, left, lineWidth, step);
            }, 40);
        }

    };
    Marquee.prototype._startScroll = function (parentNode, text, oriText) {
        if (this._boxElm && this._marqueeElm && this._displayElm) {
            this.stopScroll();
        }

        var self = this;
        var success = this._createDOM(parentNode, text, oriText);
        if (success && this._marqueeElm) {
            evm.css(this._marqueeElm, {
                display: 'block'
            });
            evm.css(this._displayElm, {
                display: 'none'
            });

            var lineWidth;
            var marqueeTextElm = evm.getElementsByClassName(this.prefix + '-text', this._marqueeElm, 'div')[0];
            this._startPauseTimeId = setTimeout(function () {
                self._timerId = setInterval(function () {
                    //todo clientWidth可能有兼容性问题
                    var lineWidth = '';
                    try {
                        var lineFirst = evm.getElementsByClassName(self.prefix + '-text', self._marqueeElm, 'span')[0];
                        lineWidth = parseFloat(evm.getStyle(lineFirst, 'width')) || lineFirst.clientWidth;
                    }
                    catch (e) {
                        lineWidth = oriText.length * self.fontSize;
                    }
                    if (lineWidth) {
                        clearInterval(self._timerId);

                        lineWidth += self.gap;
                        var left = parseFloat(evm.getStyle(self._marqueeElm, 'left'));
                        var step = -self.scrollAmount;

                        self._scrollOnce.call(self, left, lineWidth, step);
                    }
                }, 10);
            }, this.startPauseTime);
        }
    };
    Marquee.prototype._stopScroll = function () {
        var self = this;
        if (this._boxElm && this._marqueeElm && this._displayElm) {
            this._startPauseTimeId && clearTimeout(this._startPauseTimeId);
            this._timerId && clearInterval(this._timerId);
            this._scrollOnceTimerId && clearTimeout(this._scrollOnceTimerId);
            this._startPauseTimeId = null;
            this._scrollOnceTimerId = null;
            this._timerId = null;
            this._marqueeElm && (
                evm.css(this._marqueeElm, {
                    left: 0
                })
            );

            evm.css(this._marqueeElm, {
                display: 'none'
            });
            evm.css(this._displayElm, {
                display: 'block'
            });

            this._boxElm = null;
            this._marqueeElm = null;
            this._displayElm = null;
        }
    }

    //公共方法
    Marquee.prototype.startScroll = function (parentNode, text, oriText) {
        if (this.useMarqueeTag) {
            if (!parentNode) {
                return;
            }

            this._boxElm = parentNode;
            evm.html(this._boxElm, '');

            this._marqueeElm = evm.createElem('marquee', {
                className: this.prefix + '-marquee'
            }, this._boxElm);
            evm.css(this._boxElm, {
                position: 'relative',
                width: this.width + 'px',
                height: this.height + 'px',
                lineHeight: this.height + 'px',
                overflow: 'hidden',
                fontSize: this.fontSize
            });
            evm.html(this._marqueeElm, oriText);

            this._text = text;
            this._oriText = oriText;
        }
        else {
            this._startScroll(parentNode, text, oriText);
        }
    };
    Marquee.prototype.stopScroll = function () {
        if (this.useMarqueeTag) {
            if (this._boxElm) {
                if(this._marqueeElm){
                    this._marqueeElm.stop();
                    this._marqueeElm = null;
                }
                evm.html(this._boxElm, this._text);
            }
        }
        else {
            this._stopScroll();
        }
    };

    evm.namespace('evm.util.marquee');
    evm.util.marquee = function (options) {
        return new Marquee(options);
    }
})(this);
